"""Backward-compatible rubric models for synth_ai.sdk.task."""

from __future__ import annotations

from synth_ai.sdk.localapi._impl.rubrics.models import Criterion, Rubric

__all__ = ["Criterion", "Rubric"]
