"""Backward-compatible eval API namespace."""

from __future__ import annotations

from synth_ai.sdk.eval.job import EvalJob, EvalJobConfig, EvalResult

__all__ = ["EvalJob", "EvalJobConfig", "EvalResult"]
